# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorThredds.py $
# Last committed:     $Revision: 197611 $; $Author: svcdms $; $Date: 2019-07-03 10:15:11 +0200 (Wed, 03 Jul 2019) $
#
import os
import pandas as pd
import json
import sys
import getopt

import altair as alt

from datetime import datetime, timedelta

from misc.Utils import Utils
from dmsLogger.DmsLogger import DmsLogger

from hkvfewspy.utils.untangle import parse_raw
from hkvportal.io.services import dataportal
from io import StringIO

import requests

from siphon.catalog import TDSCatalog


def thredds_data(settings):
    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run MonitorThredds")

    # thredds server
    thredds_server = settings["thredds_server"]
    thredds_overview = settings["thredds_overview"]
    startdate_delta= int(settings["startdate_delta"])
    hkvservices = settings["hkvservices"]
    database = settings["database"]
    #monitor_root = settings["monitor_root"]

    tailored = 'tailored' in thredds_overview
    period_days = 1

    enddate = pd.to_datetime(pd.datetime.now().strftime('%Y-%m-%d'))
    startdate= enddate - pd.Timedelta(startdate_delta, 'D')
    
    logger.info("Run from {} until {}".format(startdate, enddate))

    # get overview at which location Sensor combination status information is expected
    df_overzicht = pd.read_csv(thredds_overview, index_col=0)
    combinations = []
    for row in df_overzicht.iterrows():
        for idx, item in enumerate(row[1]):
            if item > 0:
                parId = row[1].index[idx].split('(')[1].split(')')[0]
                pair = {"locId": row[0], "parId": parId, "factor": item}
                combinations.append(pair)

    df_monitor = pd.DataFrame.from_dict(combinations)

    df_msg = df_monitor


    thredds_report_items = []
    monitor_items = []

    for idx, item in df_monitor.iterrows():

        factor_day = float(item['factor'])

        scheme = 'HKV';
        if item['locId'] == 'BUOY1':
            scheme = 'MEETFRAME'

        url = thredds_server.replace("{SCHEME}", scheme)
        url = url.replace("{PARID}", item['parId'])
        url = url.replace("{LOCID}", item['locId'])
        
        logger.info ('Get {}'.format(url))

        response = requests.get(url)
        if response.status_code == 200:
            thredds = parse_raw(response.text)
            
            items = []
            if hasattr(thredds.catalog.dataset, 'dataset'):
                for ds in thredds.catalog.dataset.dataset:
                    
                    ts = Utils.get_timestamp_from_filename(ds['name'], '(\d*)\.\w*$', thredds=True)
                    if ts:
                        try:
                            date = pd.to_datetime(ts, format='%Y%m%d%H%M%S')
                            path, filename = os.path.split(ds['name'])
                            items.append({
                                'name': ds['name'],
                                'size': float(ds.dataSize.cdata),
                                'unit': ds.dataSize['units'],
                                'date': date
                            })

                            monitor_items.append({
                                'Sensor': item['parId'], 
                                'Locatie': item['locId'], 
                                'Bestand': filename, 
                                'Omvang': ds.dataSize.cdata, 
                                'DatumTijd': date.strftime("%Y-%m-%d %H:%M:%S")
                            })
                        except :
                            pass

            df = pd.DataFrame.from_dict(items)
          
            start = startdate
            delta = (enddate - startdate).days
            for doy in range(delta):

                end = start + pd.to_timedelta(1, unit='d')

                freq_no = 0
                valid_freq = 0

                try:

                    dt_slice = df[(df['date'] >= start) & (df['date'] < end)]

                    freq_no = dt_slice.shape[0] / (period_days * factor_day)
                    dt_gt_0 = dt_slice[dt_slice['size'] > 0]
                    valid_freq = dt_gt_0.shape[0] / dt_slice.shape[0]

                except:
                    pass

                thredds_report_items.append(
                    {
                        'Sensor': item['parId'], 
                        'Locatie': item['locId'], 
                        'Files received': '{} / {}'.format(dt_slice.shape[0], int(period_days*factor_day)), 
                        '% Received': freq_no,
                        'Valid files': valid_freq, 
                        'Date': start.strftime("%Y-%m-%d")
                    })

                start = end


    df_rep = pd.DataFrame().from_dict(thredds_report_items)

    # double check with the dataframe with location sensor factor combinations
    for row in df_monitor.iterrows():
        l = row[1]['locId']
        p = row[1]['parId']
        
        df_slice = df_rep[(df_rep['Locatie']==l) & (df_rep['Sensor']==p)]
        
        if df_slice.shape[0] == 0:    
            # sensor location combination not existing on THREDDS but should be monitored
            for d in df_rep.Date.unique():
                thredds_report_items.append({
                    'Sensor': p, 
                    'Locatie': l, 
                    'Files received': '{} / {}'.format(0, int(period_days*row[1]['factor'])), 
                    '% Received': 0,
                    'Valid files': 0, 
                    'Date': d
                })   

    df_rep = pd.DataFrame().from_dict(thredds_report_items)                 

    dp = dataportal()
    dp.setDataservice(dataservice=hkvservices)
    # run once the following line of code to generate the database
    #dp.createDatabase(database = database)
    
    # df_rep.to_csv(r"D:\test.csv")

    single_nearest = alt.selection_single(on='mouseover', empty='none')
    
    title= 'Status THREDDS Standard Data at {}'.format(pd.datetime.today().strftime('%Y-%m-%d'))
    if tailored:
        title= 'Status THREDDS Tailored Data at {}'.format(pd.datetime.today().strftime('%Y-%m-%d'))

    chart = alt.Chart(df_rep).mark_rect().encode(
        y=alt.Y('Date:N', sort='descending'),
        x=alt.X('Locatie:N', title=None),
        fill=alt.Fill(
        '% Received:Q',
        scale=alt.Scale(
            domain=[0,.85,1.00],
            range=['red','#FFDC00','#2ECC40']
        ),
        legend=alt.Legend(values=[0,0.50,1], title='Health', format='.0%')        
    ),
    stroke=alt.condition(single_nearest,alt.StrokeValue('black'), alt.StrokeValue('white')),
        tooltip=['Locatie:N',
                 'Sensor:N',
                 'Files received:N',
                 'Date:N',
                 alt.Tooltip('% Received:Q', format='.0%'),
                 alt.Tooltip('Valid files:Q', format='.0%')],
        facet=alt.Facet('Sensor:N', title=title)
    ).resolve_scale(
        x='independent'
    ).configure_scale(
        bandPaddingInner=0.05,
        bandPaddingOuter=0.05
    ).configure_axisX(
        labelAngle=-45
    ).configure_axisY(
        titleAngle=0,
        titleAlign="left",
        titleY=-3,
        titleX=-50,
    ).configure_header(
        titleFontSize=16,
        titleAnchor='start'
    ).add_selection(
        single_nearest
    )

    # export visualizatie als een 'in-memory text stream'
    export_html = StringIO()
    chart.save(export_html, format='html')

    if tailored:
        key = 'monitor_thredds_tailored_data'
        description = 'Monitor thredds tailored data'
        logger.info("{}".format('https://dmws.hkvservices.nl/dataportal/data.asmx/read2?database=dmshoutribdijk&key=monitor_thredds_tailored_data&contentType=text/html'))
        # save data in database
        dp.setEntryDatabase(database=database,
                            key='csv_thredds_tailored_data',
                            data=df_rep.to_csv(),
                            description='CSV THREDDS tailored data')
                        

    else:
        key = 'monitor_thredds_standard_data'
        description = 'Monitor thredds standard data'
        logger.info("{}".format('https://dmws.hkvservices.nl/dataportal/data.asmx/read2?database=dmshoutribdijk&key=monitor_thredds_standard_data&contentType=text/html'))
        # save data in database
        dp.setEntryDatabase(database=database,
                            key='csv_thredds_standard_data',
                            data=df_rep.to_csv(),
                            description='CSV THREDDS standard data')

    # save chart in database
    dp.setEntryDatabase(database=database,
                        key=key,
                        data=export_html.getvalue(),
                        description=description)
    
    logger.info("End of MonitorThredds")

def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorThredds.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorThredds.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        # settings_file = "../connections/monitor_status_locs_sensors.json"
        settings = Utils.read_thredds_settings(settings_file)
        thredds_data(settings)
    except FileNotFoundError:
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
    # fire.Fire()
